package com.ywt.alipaympapi;

import com.alipay.easysdk.factory.Factory;
import com.alipay.easysdk.kernel.Config;
import com.ywt.alipaympapi.core.utils.BizUtil;
import com.ywt.alipaympapi.core.utils.Checker;
import com.ywt.alipaympapi.models.Constants;
import com.ywt.alipaympapi.models.auth.AlipayMpCfg;
import com.ywt.alipaympapi.models.enums.TerminalEnum;
import com.ywt.core.exception.AppMessageException;
import com.ywt.rpc.core.RestServerStarter;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.EnableAspectJAutoProxy;

@SpringBootApplication
@EnableAspectJAutoProxy
public class EmptyApplication {

    public static void main(String[] args) {
        SpringApplication.run(EmptyApplication.class, args);
        String serviceName = "com.ywt.AlipayMpRestService";
        System.out.println("--- RegisterService: " + serviceName);
        RestServerStarter.build(null).registry(serviceName, null, "支付宝小程序服务");
        initAlipayEasySdk();
    }

    /**
     * 初始化 alipay-easy-sdk
     * TODO: 后续扩展多医院需要将 pk & apk 配置化
     */
    private static void initAlipayEasySdk() {
        Config config = new Config();
        config.protocol = "https";
        config.gatewayHost = "openapi.alipay.com";
        config.signType = "RSA2";

        try {
            AlipayMpCfg cfg = BizUtil.getCfg(TerminalEnum.NFYYBYFY_ALIPAY_MP.getValue());
            config.appId = Checker.getStringValue(cfg.getAppId());
            // 为避免私钥随源码泄露，推荐从文件中读取私钥字符串而不是写入源码中
            config.merchantPrivateKey = Checker.getStringValue(cfg.getMerchantPrivateKey());
            // 注：如果采用非证书模式，则无需赋值上面的三个证书路径，改为赋值如下的支付宝公钥字符串即可
            config.alipayPublicKey = Checker.getStringValue(cfg.getPublicKey());
            Factory.setOptions(config);
        } catch (AppMessageException e) {
            System.out.println("EmptyApplication#initAlipayEasySdk(): 获取配置失败" + e.getMessage());
            e.printStackTrace();
        }


    }
}
